// qtractorMidiControl.cpp
//
/****************************************************************************
   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*****************************************************************************/

#include "qtractorMidiControl.h"

#include "qtractorMidiControlTestForm.h"


//----------------------------------------------------------------------
// class qtractorMidiControlObserver -- MIDI controller observers.
//

QList<qtractorMidiControlObserver *>
qtractorMidiControlObserver::g_midiControlObservers;


// Constructor.
qtractorMidiControlObserver::qtractorMidiControlObserver (
	qtractorSubject *pSubject,
	unsigned short iChannel,
	unsigned short iController )
	: qtractorObserver(pSubject),
		m_iChannel(iChannel),
		m_iController(iController)
{
	g_midiControlObservers.append(this);
}


// Destructor.
qtractorMidiControlObserver::~qtractorMidiControlObserver (void)
{
	g_midiControlObservers.removeAll(this);
}


// MIDI controller dispatcher (static).
void qtractorMidiControlObserver::dispatch (
	unsigned short iChannel, unsigned short iController, unsigned short iValue )
{
	QListIterator<qtractorMidiControlObserver *> it(g_midiControlObservers);
	while (it.hasNext()) {
		qtractorMidiControlObserver *pMidiObserver = it.next();
		if (pMidiObserver->channel() == iChannel &&
			pMidiObserver->controller() == iController) {
			pMidiObserver->setValue(float(iValue) / 127.0f);
		}
	}
}


// Updater.
void qtractorMidiControlObserver::update (void)
{
	qtractorMidiControlTestForm *pTestForm
		= qtractorMidiControlTestForm::getInstance();
	if (pTestForm) {
		pTestForm->sendController(
			m_iChannel, m_iController, int(127.0f * value()));
	}
}


// end of qtractorMidiControl.cpp
