// qtractorMidiControlTestForm.h
//
/****************************************************************************
   Copyright (C) 2005-2009, rncbc aka Rui Nuno Capela. All rights reserved.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*****************************************************************************/

#ifndef __qtractorMidiControlTestForm_h
#define __qtractorMidiControlTestForm_h

#include "ui_qtractorMidiControlTestForm.h"

#include <alsa/asoundlib.h>

// Forward declarations.
class qtractorSubject;

class qtractorCheckBoxObserver;
class qtractorSpinBox1Observer;
class qtractorSpinBox2Observer;
class qtractorSliderObserver;

class qtractorMidiControlObserver;

class qtractorMidiControlInputThread;


//----------------------------------------------------------------------------
// qtractorMidiControlTestForm -- UI wrapper form.

class qtractorMidiControlTestForm : public QDialog
{
	Q_OBJECT

public:

	// Constructor.
	qtractorMidiControlTestForm(QWidget *pParent = 0, Qt::WindowFlags wflags = 0);
	// Destructor.
	~qtractorMidiControlTestForm();

	// Pseudo-singleton reference.
	static qtractorMidiControlTestForm *getInstance();

	// ALSA client descriptor accessor.
	snd_seq_t *alsaSeq() const;
	int alsaClient() const;
	int alsaPort() const;

	// MIDI event capture method.
	void capture(snd_seq_event_t *pEv);

	// Direct MIDI controller common helper.
	void sendController(unsigned short iChannel,
		unsigned short iController, unsigned short iValue = 0) const;

protected slots:

	void booleanCheckBoxChanged(bool);

	void integerSpinBoxChanged(const QString&);

	void linearSpinBoxChanged(const QString&);
	void linearSliderChanged(int);

	void logSpinBoxChanged(const QString&);
	void logSliderChanged(int);

	void timerSlot();

protected:

	// Custom event handler.
	void customEvent(QEvent *pEvent);

	// Handle MIDI controller event.
	void receiveController(unsigned short iChannel,
		unsigned short iController, unsigned short iValue);

private:

	// The Qt-designer UI struct...
	Ui::qtractorMidiControlTestForm m_ui;

	// Pseudo-singleton reference.
	static qtractorMidiControlTestForm *g_pTestForm;

	// Instance variables.
	snd_seq_t *m_pAlsaSeq;
	int        m_iAlsaClient;
	int        m_iAlsaPort;

	// Name says it all.
	qtractorMidiControlInputThread  *m_pInputThread;

	// Subjects.
	qtractorSubject *m_pBooleanSubject;
	qtractorSubject *m_pIntegerSubject;
	qtractorSubject *m_pLinearSubject;
	qtractorSubject *m_pLogSubject;

	// MIDI Observers.
	qtractorMidiControlObserver *m_pBooleanMidiObserver;
	qtractorMidiControlObserver *m_pIntegerMidiObserver;
	qtractorMidiControlObserver *m_pLinearMidiObserver;
	qtractorMidiControlObserver *m_pLogMidiObserver;
};


#endif	// __qtractorMidiControlTestForm_h


// end of qtractorMidiControlTestForm.h
